<?php
/* --------------------------------------------------------------
   Amazon Pay  V3.1.0
   AlkimAmazonTransactions.class.php 2017-02-10

   alkim media
   http://www.alkim.de

   patworx multimedia GmbH
   http://www.patworx.de/

   Released under the GNU General Public License
   --------------------------------------------------------------
*/
?><?php


class AlkimAmazonTransactions
{
    public static function authorize($orderRef, $amount, $timeout=1440, $comment='')
    {
        $requestParameters = array();
        $requestParameters['authorization_amount'] = $amount;
        $requestParameters['amazon_order_reference_id'] = $orderRef;
        $requestParameters['transaction_timeout'] = $timeout;
        $requestParameters['currency_code'] = self::getOrderRefCurrency($orderRef);
        $requestParameters['merchant_id'] = AWS_MERCHANT_ID;
        $requestParameters['soft_descriptor'] = $comment;
        $requestParameters['authorization_reference_id'] = self::getNextAuthRef($orderRef);
        if (MODULE_PAYMENT_AM_APA_PROVOCATION == 'hard_decline' && MODULE_PAYMENT_AM_APA_MODE == 'sandbox') {
            $requestParameters['seller_authorization_note'] = '{"SandboxSimulation": {"State":"Declined", "ReasonCode":"AmazonRejected"}}';
        }

        if (MODULE_PAYMENT_AM_APA_PROVOCATION == 'soft_decline' && MODULE_PAYMENT_AM_APA_MODE == 'sandbox') {
            $requestParameters['seller_authorization_note'] = '{"SandboxSimulation": {"State":"Declined", "ReasonCode":"InvalidPaymentMethod", "PaymentMethodUpdateTimeInMins":1}}';
        }

        $response = AlkimAmazonHandler::call('authorize', $requestParameters);

        $authId = '';
        $details = $response["AuthorizeResult"]["AuthorizationDetails"];
        $authId = $details["AmazonAuthorizationId"];
        if ($authId != '') {
            $sqlArr = array(
                                    'amz_tx_order_reference'    =>  $orderRef,
                                    'amz_tx_type'   => 'auth',
                                    'amz_tx_time' => time(),
                                    'amz_tx_expiration'=>strtotime($details["ExpirationTimestamp"]),
                                    'amz_tx_amount'=>$amount,
                                    'amz_tx_status'=>$details["AuthorizationStatus"]["State"],
                                    'amz_tx_reference'=>$details["AuthorizationReferenceId"],
                                    'amz_tx_amz_id'=>$authId,
                                    'amz_tx_last_change'=>time(),
                                    'amz_tx_last_update'=>time(),
                                    'amz_tx_merchant_id'=>AWS_MERCHANT_ID,
                                    'amz_tx_mode'=>MODULE_PAYMENT_AM_APA_MODE


                                );

            xtc_db_perform('amz_transactions', $sqlArr);
        }
        AlkimAmazonHandler::log('api', array('orderRef'=>$orderRef, 'txnId'=> $authId, 'summary'=>'Authorization'), print_r($requestParameters, true), print_r($response, true));
        return $response;
    }


    public static function capture($authId, $amount)
    {
        if ($authId) {
            self::refreshAuthorization($authId, true);
            $orderRef = self::getOrderRefFromAmzId($authId);

            $requestParameters = array();
            $requestParameters['merchant_id'] = AWS_MERCHANT_ID;
            $requestParameters['amazon_order_reference_id'] = $orderRef;
            $requestParameters['amazon_authorization_id'] = $authId;
            $requestParameters['capture_amount'] = $amount;
            $requestParameters['currency_code'] = self::getOrderRefCurrency($orderRef);
            $requestParameters['capture_reference_id'] = self::getNextCaptureRef($orderRef);

            if (MODULE_PAYMENT_AM_APA_PROVOCATION == 'capture_decline' && MODULE_PAYMENT_AM_APA_MODE == 'sandbox') {
                $requestParameters['seller_capture_note'] = '{"SandboxSimulation":{"State":"Declined", "ReasonCode":"AmazonRejected"}}';
            }

            $captureId = '';
            $response = AlkimAmazonHandler::call('capture', $requestParameters);
            $details = $response["CaptureResult"]["CaptureDetails"];

            $captureId = $details["AmazonCaptureId"];
            $sqlArr = array(
                                        'amz_tx_order_reference'    =>  $orderRef,
                                        'amz_tx_type'   => 'capture',
                                        'amz_tx_time' => time(),
                                        'amz_tx_expiration'=>0,
                                        'amz_tx_amount'=>$amount,
                                        'amz_tx_status'=>$details["CaptureStatus"]["State"],
                                        'amz_tx_reference'=>$details["CaptureReferenceId"],
                                        'amz_tx_amz_id'=>$captureId,
                                        'amz_tx_last_change'=>time(),
                                        'amz_tx_last_update'=>time(),
                                        'amz_tx_merchant_id'=>AWS_MERCHANT_ID,
                                        'amz_tx_mode'=>MODULE_PAYMENT_AM_APA_MODE
                                    );

            $q = "SELECT * FROM amz_transactions WHERE amz_tx_amz_id = '".xtc_db_input($captureId)."'";
            $rs = xtc_db_query($q);

            if (xtc_db_num_rows($rs)==0) {
                xtc_db_perform('amz_transactions', $sqlArr);
                $txId = xtc_db_insert_id();
                if($sqlArr["amz_tx_status"] == 'Completed'){
                    $orderTotal = round(AlkimAmazonTransactions::getOrderRefTotal($orderRef), 2);
                    if(round(AlkimAmazonHandler::getOrderCapturedAmount($orderRef), 2) >= $orderTotal){
                        AlkimAmazonTransactions::setOrderStatusCaptured($orderRef, $txId);
                        self::refreshAuthorization($authId);
                        if(AMZ_CLOSE_ORDER_ON_COMPLETE_CAPTURE == 'True'
                            &&
                           !self::hasOrderPendingCaptures($orderRef)
                            &&
                           !self::hasOrderOpenOrPendingAuthorizations($orderRef)) {
                            self::closeOrder($orderRef);
                        }
                    }else{
                        AlkimAmazonTransactions::setOrderStatusPartiallyCaptured($orderRef, $txId);
                    }
                }
            }
            AlkimAmazonHandler::log('api', array('orderRef'=>$orderRef, 'txnId'=> $captureId, 'summary'=>'Capture'), print_r($requestParameters, true), print_r($response, true));
            return $response;
        } else {
            //TODO: No auth for capture
        }
    }

    public static function refund($captureId, $amount)
    {
        $orderRef = self::getOrderRefFromAmzId($captureId);

        $requestParameters = array();
        $requestParameters["amazon_capture_id"] = $captureId;
        $requestParameters["refund_reference_id"] = self::getNextRefundRef($orderRef);
        $requestParameters["refund_amount"] = $amount;
        $requestParameters["currency_code"] = self::getOrderRefCurrency($orderRef);

        $response = AlkimAmazonHandler::call('refund', $requestParameters);
        $details = $response["RefundResult"]["RefundDetails"];
        $refundId = $details["AmazonRefundId"];
        $sqlArr = array(
                                        'amz_tx_order_reference'    =>  $orderRef,
                                        'amz_tx_type'   => 'refund',
                                        'amz_tx_time' => time(),
                                        'amz_tx_expiration'=>0,
                                        'amz_tx_amount'=>$amount,
                                        'amz_tx_status'=>$details["RefundStatus"]["State"],
                                        'amz_tx_reference'=>$details["RefundReferenceId"],
                                        'amz_tx_amz_id'=>$refundId,
                                        'amz_tx_last_change'=>time(),
                                        'amz_tx_last_update'=>time(),
                                        'amz_tx_merchant_id'=>AWS_MERCHANT_ID,
                                        'amz_tx_mode'=>MODULE_PAYMENT_AM_APA_MODE
                                    );

        xtc_db_perform('amz_transactions', $sqlArr);
        AlkimAmazonHandler::log('api', array('orderRef'=>$orderRef, 'txnId'=> $refundId, 'summary'=>'Refund'), print_r($requestParameters, true), print_r($response, true));
        return $response;
    }

    public static function refreshOrderReference($orderRef)
    {
        $q = "SELECT * FROM amz_transactions WHERE amz_tx_amz_id = '".xtc_db_input($orderRef)."'";
        $rs = xtc_db_query($q);
        $r = xtc_db_fetch_array($rs);
        $oldStatus = (string)$r["amz_tx_status"];

        $orderReferenceDetails = AlkimAmazonHandler::call('GetOrderReferenceDetails', array('amazon_order_reference_id'=>$orderRef));
        $details = $orderReferenceDetails["GetOrderReferenceDetailsResult"]["OrderReferenceDetails"];

        $sqlArr = array(
                                            'amz_tx_status' => (string)$details["OrderReferenceStatus"]["State"],
                                            'amz_tx_last_change' =>strtotime((string)$details["OrderReferenceStatus"]["LastUpdateTimestamp"]),
                                            'amz_tx_last_update'=>time()
                                        );
        xtc_db_perform('amz_transactions', $sqlArr, 'update', " amz_tx_amz_id = '".xtc_db_input($orderRef)."'");
                 /*
                 if($oldStatus == 'Suspended' && $sqlArr["amz_tx_status"] == 'Open'){
                    self::reAuthByOrderRef($orderRef);
                 }
                 */
                 if ($sqlArr["amz_tx_status"] == 'Open') {
                     AlkimAmazonTransactions::doAuthorizationAfterDecline($orderRef);
                 }
    }

    public static function closeOrder($orderRef)
    {
        $requestParameters = array();
        $requestParameters['amazon_order_reference_id'] = $orderRef;
        $response = AlkimAmazonHandler::call('closeOrderReference', $requestParameters);
        AlkimAmazonHandler::log('api', array('orderRef'=>$orderRef, 'summary'=>'closeOrder'), print_r($requestParameters, true), print_r($response, true));
    }

    public static function cancelOrder($orderRef)
    {
        $requestParameters = array();
        $requestParameters['amazon_order_reference_id'] = $orderRef;
        $response = AlkimAmazonHandler::call('cancelOrderReference', $requestParameters);
        AlkimAmazonHandler::log('api', array('orderRef'=>$orderRef, 'summary'=>'cancelOrder'), print_r($requestParameters, true), print_r($response, true));
    }

    public static function refreshRefund($refundId)
    {
        $requestParameters = array();
        $requestParameters['amazon_refund_id'] = $refundId;
        $response = AlkimAmazonHandler::call('getRefundDetails', $requestParameters);

        $details = $response["GetRefundDetailsResult"]["RefundDetails"];
        $status = (string)$details["RefundStatus"]["State"];

        $sqlArr = array(
                                        'amz_tx_status' => $status,
                                        'amz_tx_last_change' =>strtotime((string)$details["RefundStatus"]["LastUpdateTimestamp"]),
                                        'amz_tx_last_update'=>time()
                                    );
        xtc_db_perform('amz_transactions', $sqlArr, 'update', " amz_tx_amz_id = '".xtc_db_input($refundId)."'");
    }

    public static function refreshCapture($captureId)
    {
        $requestParameters = array();
        $requestParameters['amazon_capture_id'] = $captureId;
        $response = AlkimAmazonHandler::call('getCaptureDetails', $requestParameters);

        $details = $response["GetCaptureDetailsResult"]["CaptureDetails"];

        $sqlArr = array(
                                        'amz_tx_status' => (string)$details["CaptureStatus"]["State"],
                                        'amz_tx_last_change' =>strtotime((string)$details["CaptureStatus"]["LastUpdateTimestamp"]),
                                        'amz_tx_amount_refunded' =>(float)$details["RefundedAmount"]["Amount"],
                                        'amz_tx_last_update'=>time()
                                    );

        xtc_db_perform('amz_transactions', $sqlArr, 'update', " amz_tx_amz_id = '".xtc_db_input($captureId)."'");

        if ($sqlArr['amz_tx_status'] == 'Completed') {
            $q = "SELECT * FROM amz_transactions WHERE amz_tx_type = 'capture' AND amz_tx_amz_id = '".xtc_db_input($captureId)."'";
            $rs = xtc_db_query($q);
            if (xtc_db_num_rows($rs) == 1) {
                $r = xtc_db_fetch_array($rs);
                $orderReference = $r["amz_tx_order_reference"];
                $orderTotal = round(AlkimAmazonTransactions::getOrderRefTotal($r["amz_tx_order_reference"]), 2);
                if($r["amz_tx_admin_informed"] == 0){
                    if(round(AlkimAmazonHandler::getOrderCapturedAmount($orderReference), 2) >= $orderTotal){
                        AlkimAmazonTransactions::setOrderStatusCaptured($orderReference, $r["amz_tx_id"]);
                    }else{
                        AlkimAmazonTransactions::setOrderStatusPartiallyCaptured($orderReference, $r["amz_tx_id"]);
                    }
                }

                if (        AMZ_CLOSE_ORDER_ON_COMPLETE_CAPTURE == 'True'
                                &&
                            !self::hasOrderPendingCaptures($orderReference)
                                &&
                            AlkimAmazonHandler::getOrderCapturedAmount($orderReference) >= $orderTotal
                                &&
                            !self::hasOrderOpenOrPendingAuthorizations($orderReference)) {
                    self::closeOrder($r["amz_tx_order_reference"]);
                }
            }
        } elseif ($sqlArr['amz_tx_status'] == 'Declined') {
            AlkimAmazonHandler::captureDeclinedAction($captureId);
        }
    }

    public static function hasOrderPendingCaptures($orderReference)
    {
        $q = "SELECT * FROM amz_transactions WHERE amz_tx_type = 'capture' AND amz_tx_status = 'Pending' AND amz_tx_order_reference = '".xtc_db_input($orderReference)."'";
        $rs = xtc_db_query($q);
        return (xtc_db_num_rows($rs)>0);
    }

    public static function hasOrderOpenOrPendingAuthorizations($orderReference)
    {
        $q = "SELECT * FROM amz_transactions WHERE amz_tx_type = 'auth' AND amz_tx_status IN ('Pending', 'Open') AND amz_tx_order_reference = '".xtc_db_input($orderReference)."'";
        $rs = xtc_db_query($q);
        return (xtc_db_num_rows($rs)>0);
    }

    public static function refreshAuthorization($authId, $noCapture = false)
    {
        $requestParameters = array();
        $requestParameters['amazon_authorization_id'] = $authId;
        $response = AlkimAmazonHandler::call('getAuthorizationDetails', $requestParameters);

        $details = $response["GetAuthorizationDetailsResult"]["AuthorizationDetails"];
        $status = (string)$details["AuthorizationStatus"]["State"];

        $address = $details["AuthorizationBillingAddress"];
        if (!empty($address)) {
            $addressArr = AlkimAmazonHandler::getAddressArrayFromAmzAddress($address);
            $q = "SELECT * FROM amz_transactions WHERE amz_tx_amz_id = '".xtc_db_input($authId)."'";
            $txRs = xtc_db_query($q);
            $txR = xtc_db_fetch_array($txRs);

            $sqlArr = array(
                                            'billing_name'=>$addressArr["name"],
                                            'billing_firstname'=>$addressArr["firstname"],
                                            'billing_lastname'=>$addressArr["lastname"],
                                            'billing_company'=>$addressArr["company"],
                                            'billing_suburb'=>$addressArr["suburb"],
                                            'billing_street_address'=>$addressArr["street_address"],
                                            'billing_postcode'=>$addressArr["postcode"],
                                            'billing_city'=>$addressArr["city"],
                                            'billing_country'=>$addressArr["country"]["title"],
                                            'billing_address_format_id'=>5,
                                            'billing_country_iso_code_2'=>$addressArr["country"]["iso_code_2"]
                                        );

            xtc_db_perform(TABLE_ORDERS, $sqlArr, 'update', " amazon_order_id = '".xtc_db_input($txR["amz_tx_order_reference"])."'");
        }
        $sqlArr = array(
                                        'amz_tx_status' => $status,
                                        'amz_tx_last_change' =>strtotime((string)$details["AuthorizationStatus"]["LastUpdateTimestamp"]),
                                        'amz_tx_last_update'=>time()
                                    );

        xtc_db_perform('amz_transactions', $sqlArr, 'update', " amz_tx_amz_id = '".xtc_db_input($authId)."'");

        if ($status == 'Open') {
            if (AMZ_CAPTURE_MODE == 'after_auth' && !$noCapture) {
                if (AlkimAmazonHandler::isAutomaticAuth($authId)) {
                    AlkimAmazonTransactions::capture($authId, AlkimAmazonHandler::getAuthAmount($authId));
                }
            }


            $q = "SELECT amz_tx_admin_informed FROM amz_transactions WHERE amz_tx_amz_id = '".xtc_db_input($authId)."'";
            $rs = xtc_db_query($q);
            $r = xtc_db_fetch_array($rs);
            if ($r["amz_tx_admin_informed"]==0) {
                AlkimAmazonHandler::handleOpenAuth($authId);
            }


            $q = "UPDATE amz_transactions SET amz_tx_admin_informed = 1 WHERE amz_tx_amz_id = '".xtc_db_input($authId)."'";
            xtc_db_query($q);
        }

        if ($status == 'Declined') {
            $reason = (string)$details["AuthorizationStatus"]["ReasonCode"];
            if ($reason == 'AmazonRejected') {
                $orderRef = AlkimAmazonTransactions::getOrderRefFromAmzId($authId);
                AlkimAmazonTransactions::cancelOrder($orderRef);
            }
            AlkimAmazonHandler::authorizationDeclinedAction($authId, $reason);
        }
    }

    public static function intelligentRefresh($r)
    {
        switch ($r["amz_tx_type"]) {
                    case 'refund':
                        self::refreshRefund($r["amz_tx_amz_id"]);
                        break;
                    case 'capture':
                        self::refreshCapture($r["amz_tx_amz_id"]);
                        break;
                   case 'auth':
                        self::refreshAuthorization($r["amz_tx_amz_id"]);
                        break;
                    case 'order_ref':
                        self::refreshOrderReference($r["amz_tx_amz_id"]);
                        break;

                }
    }

    public static function doAuthorizationAfterDecline($orderRef)
    {
        $q = "SELECT * FROM amz_transactions
                    WHERE
                        amz_tx_order_reference = '".xtc_db_input($orderRef)."'
                            AND
                        amz_tx_type = 'auth'
                            AND
                        amz_tx_status = 'Declined'
                    ORDER BY
                        amz_tx_id DESC
                    LIMIT 1";
        $rs = xtc_db_query($q);
        if ($r = xtc_db_fetch_array($rs)) {
            $q = "SELECT * FROM amz_transactions
                        WHERE
                            amz_tx_order_reference = '".xtc_db_input($orderRef)."'
                                AND
                            amz_tx_type = 'auth'
                                AND
                            amz_tx_status != 'Declined'";
            $rs = xtc_db_query($q);
            if (xtc_db_num_rows($rs)==0) {
                self::authorize($orderRef, $r["amz_tx_amount"]);
            }
        }
    }

    public static function getAuthorizationsForCapture($orderRef)
    {
        $q = "SELECT * FROM amz_transactions WHERE amz_tx_status = 'Open' AND amz_tx_type = 'auth' AND amz_tx_order_reference = '".xtc_db_input($orderRef)."'";
        $rs = xtc_db_query($q);
        $ret = array();
        while ($r = xtc_db_fetch_array($rs)) {
            $ret[] = $r;
        }
        return $ret;
    }

    public static function captureTotalFromAuth($authId)
    {
        $q = "SELECT * FROM amz_transactions WHERE amz_tx_type='auth' AND amz_tx_amz_id = '".xtc_db_input($authId)."'";
        $rs = xtc_db_query($q);
        $r = xtc_db_fetch_array($rs);
        if ($r) {
            return self::capture($authId, $r["amz_tx_amount"]);
        } else {
            return false;
        }
    }

    public static function reAuthByOrderRef($orderRef)
    {
        $q = "SELECT * FROM amz_transactions WHERE amz_tx_status = 'Declined' AND amz_tx_type = 'auth' AND amz_tx_order_reference = '".xtc_db_input($orderRef)."' ORDER BY amz_tx_id DESC LIMIT 1";
        $rs = xtc_db_query($q);
        if ($r = xtc_db_fetch_array($rs)) {
            self::reAuth($r["amz_tx_amz_id"]);
        }
    }

    public static function reAuth($authId)
    {
        $q = "SELECT * FROM amz_transactions WHERE amz_tx_amz_id = '".xtc_db_input($authId)."'";
        $rs = xtc_db_query($q);
        if ($r = xtc_db_fetch_array($rs)) {
            self::authorize($r["amz_tx_order_reference"], $r["amz_tx_amount"]);
        }
    }

    public static function getNextAuthRef($orderRef)
    {
        return self::getNextRef($orderRef, 'auth');
    }

    public static function getNextCaptureRef($orderRef)
    {
        return self::getNextRef($orderRef, 'capture');
    }

    public static function getNextRefundRef($orderRef)
    {
        return self::getNextRef($orderRef, 'refund');
    }

    public static function getNextRef($orderRef, $type)
    {
        $lastId = 0;
        $prefix = substr($type, 0, 1);
        $q = "SELECT * FROM amz_transactions WHERE amz_tx_type='".$type."' AND amz_tx_order_reference = '".xtc_db_input($orderRef)."' ORDER BY amz_tx_id DESC LIMIT 1";
        $rs = xtc_db_query($q);
        if ($r = xtc_db_fetch_array($rs)) {
            $lastId = (int)str_replace($orderRef.'-'.$prefix, '', $r["amz_tx_reference"]);
        }
        $newId = $lastId+1;
        return $orderRef.'-'.$prefix.str_pad($newId, 2, '0', STR_PAD_LEFT);
    }

    public static function getOrderRefFromAmzId($amzId)
    {
        $q = "SELECT * FROM amz_transactions WHERE amz_tx_amz_id = '".xtc_db_input($amzId)."'";
        $rs = xtc_db_query($q);
        $r = xtc_db_fetch_array($rs);
        return $r["amz_tx_order_reference"];
    }
    public static function fastAuth($orderRef, $amount, $comment='')
    {
        ob_start();
        $response = self::authorize($orderRef, $amount, 0, $comment);
        $details = $response["AuthorizeResult"]["AuthorizationDetails"];
        $authId = $details["AmazonAuthorizationId"];
        self::refreshAuthorization($authId);
        ob_end_clean();
        return $response;
    }

    public static function setOrderStatusAuthorized($orderRef)
    {
        $oid = self::getOrdersIdFromOrderRef($orderRef);
        if ($oid) {
            if (AMZ_CAPTURE_MODE=='after_shipping' && MODULE_PAYMENT_AM_APA_STATUS == 'True' && self::getOrderStatus($oid) == AMZ_SHIPPED_STATUS) {
                $q = "SELECT * FROM amz_transactions
                        WHERE
                            amz_tx_order_reference = '".xtc_db_input($orderRef)."'
                                AND
                            amz_tx_type = 'auth'
                                AND
                            amz_tx_status = 'Open'
                        ORDER BY amz_tx_id DESC";
                $rs = xtc_db_query($q);
                if ($r = xtc_db_fetch_array($rs)) {
                    if (AlkimAmazonHandler::isFullAuth($r["amz_tx_amz_id"])) {
                        AlkimAmazonTransactions::capture($r["amz_tx_amz_id"], AlkimAmazonHandler::getAuthAmount($r["amz_tx_amz_id"]));
                    }
                }
            }
            $newStatus = MODULE_PAYMENT_AM_APA_ORDER_STATUS_OK;
            $comment = 'Amazon Pay';
            self::setOrderStatus($oid, $newStatus, $comment);
        } else {
            $_SESSION["amzSetStatusAuthorized"][] = $orderRef;
        }
    }

    public static function setOrderStatusSoftDecline($orderRef, $txId = null)
    {
        self::setOrderStatusMaster($orderRef, AMZ_ORDER_STATUS_SOFT_DECLINE, "", $txId);
    }

    public static function setOrderStatusHardDecline($orderRef, $txId = null)
    {
        self::setOrderStatusMaster($orderRef, AMZ_STATUS_HARDDECLINE, "", $txId);
    }

    public static function setOrderStatusCaptured($orderRef, $txId = null)
    {
        if(AMZ_ORDER_STATUS_CAPTURED != -1){
            self::setOrderStatusMaster($orderRef, AMZ_ORDER_STATUS_CAPTURED, "amzSetStatusCaptured", $txId);
        }
    }

    public static function setOrderStatusPartiallyCaptured($orderRef, $txId = null)
    {
        if(AMZ_ORDER_STATUS_PARTIALLY_CAPTURED != -1){
            self::setOrderStatusMaster($orderRef, AMZ_ORDER_STATUS_PARTIALLY_CAPTURED, "amzSetStatusPartiallyCaptured", $txId);
        }
    }

    public static function setOrderStatusMaster($orderRef, $status, $arrayKeyForLaterProcessing, $txId = null){
        $oid = self::getOrdersIdFromOrderRef($orderRef);
        if ($oid) {
            $comment = 'Amazon Pay';
            self::setOrderStatus($oid, $status, $comment);
            if(!empty($txId)){
                $q = "UPDATE amz_transactions SET amz_tx_admin_informed = 1 WHERE amz_tx_id = '".xtc_db_input($txId)."'";
                xtc_db_query($q);
            }
        } else {
            $_SESSION[$arrayKeyForLaterProcessing][] = $orderRef;
        }
    }



    public static function getOrderStatus($oid)
    {
        $q = "SELECT orders_status FROM ".TABLE_ORDERS." WHERE orders_id = ".(int)$oid;
        $rs = xtc_db_query($q);
        $r = xtc_db_fetch_array($rs);
        return (int)$r["orders_status"];
    }
    public static function setOrderStatus($oid, $status, $comment)
    {
        $sql_data_array = array('orders_id' => $oid,
                           'orders_status_id' => $status,
                           'date_added' => 'now()',
                           'customer_notified' => 0,
                           'comments' => $comment);
        xtc_db_perform(TABLE_ORDERS_STATUS_HISTORY, $sql_data_array);
        $q = "UPDATE ".TABLE_ORDERS." SET orders_status = ".(int)$status." WHERE orders_id = ".(int)$oid;
        xtc_db_query($q);
    }

    public static function getOrdersIdFromOrderRef($orderRef)
    {
        $q = "SELECT orders_id FROM ".TABLE_ORDERS." WHERE amazon_order_id = '".xtc_db_input($orderRef)."'";
        $rs = xtc_db_query($q);
        $r = xtc_db_fetch_array($rs);
        return $r["orders_id"];
    }

    public static function getOrderTotalFromOrderRef($orderRef)
    {
        $oid = self::getOrdersIdFromOrderRef($orderRef);
        $total = self::getOrderTotal($oid);
        return $total;
    }

    public static function getOrderRefTotal($orderRef)
    {
        $q = "SELECT * FROM amz_transactions WHERE amz_tx_order_reference = '".xtc_db_input($orderRef)."' AND amz_tx_type = 'order_ref'";
        $rs = xtc_db_query($q);
        $r = xtc_db_fetch_array($rs);
        return (float)$r["amz_tx_amount"];
    }
    
    public static function getOrderRefCurrency($orderRef)
    {
        $q = "SELECT amz_tx_currency FROM amz_transactions WHERE amz_tx_order_reference = '".xtc_db_input($orderRef)."' AND amz_tx_type = 'order_ref'";
        $rs = xtc_db_query($q);
        $r = xtc_db_fetch_array($rs);
        return (empty($r["amz_tx_currency"])?'EUR':$r["amz_tx_currency"]);
    }

    public static function getOrderTotal($oid)
    {
        $q = "SELECT * FROM ".TABLE_ORDERS_TOTAL." WHERE orders_id = ".(int)$oid." AND class = 'ot_total'";
        $rs = xtc_db_query($q);
        $r = xtc_db_fetch_array($rs);
        return (float)$r["value"];
    }
}
